@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- This module re-exports the most part of the library functionality.
 -- But there are modules that must be imported explicitly.
@@ -35,7 +35,7 @@ module Simulation.Aivika
         module Simulation.Aivika.Processor.RoundRobbin,
         module Simulation.Aivika.QueueStrategy,
         module Simulation.Aivika.Ref,
-        module Simulation.Aivika.Resource,
+        module Simulation.Aivika.Resource.Base,
         module Simulation.Aivika.Results,
         module Simulation.Aivika.Results.Locale,
         module Simulation.Aivika.Results.IO,
@@ -77,7 +77,7 @@ import Simulation.Aivika.Processor.Random
 import Simulation.Aivika.Processor.RoundRobbin
 import Simulation.Aivika.QueueStrategy
 import Simulation.Aivika.Ref
-import Simulation.Aivika.Resource
+import Simulation.Aivika.Resource.Base
 import Simulation.Aivika.Results
 import Simulation.Aivika.Results.Locale
 import Simulation.Aivika.Results.IO
