@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- It models an activity that can be utilised. The activity is similar to a 'Server'
 -- but destined for simulation within 'Net' computation.
@@ -56,8 +56,8 @@ module Simulation.Aivika.Activity
         -- * Basic Signals
         activityUtilising,
         activityUtilised,
-        activityPreempting,
-        activityReentering,
+        activityPreemptionBeginning,
+        activityPreemptionEnding,
         -- * Overall Signal
         activityChanged_) where
 
@@ -72,7 +72,6 @@ import Simulation.Aivika.Simulation
 import Simulation.Aivika.Dynamics
 import Simulation.Aivika.Internal.Event
 import Simulation.Aivika.Signal
-import Simulation.Aivika.Resource
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Net
@@ -106,9 +105,9 @@ data Activity s a b =
              -- ^ A signal raised when starting to utilise the activity.
              activityUtilisedSource :: SignalSource (a, b),
              -- ^ A signal raised when the activity has been utilised.
-             activityPreemptingSource :: SignalSource a,
+             activityPreemptionBeginningSource :: SignalSource a,
              -- ^ A signal raised when the utilisation was preempted.
-             activityReenteringSource :: SignalSource a
+             activityPreemptionEndingSource :: SignalSource a
              -- ^ A signal raised when the utilisation was proceeded after it had been preempted earlier.
            }
 
@@ -181,8 +180,8 @@ newPreemptibleStateActivity preemptible provide state =
                        activityPreemptionTimeRef = r6,
                        activityUtilisingSource = s1,
                        activityUtilisedSource = s2,
-                       activityPreemptingSource = s3,
-                       activityReenteringSource = s4 }
+                       activityPreemptionBeginningSource = s3,
+                       activityPreemptionEndingSource = s4 }
 
 -- | Return a network computation for the specified activity.
 --
@@ -237,7 +236,7 @@ activityProcessPreempting act s a =
             handleSignal (processPreemptionBeginning pid) $ \() ->
             do t0 <- liftDynamics time
                liftIO $ writeIORef r0 t0
-               triggerSignal (activityPreemptingSource act) a
+               triggerSignal (activityPreemptionBeginningSource act) a
      h2  <- liftEvent $
             handleSignal (processPreemptionEnding pid) $ \() ->
             do t0 <- liftIO $ readIORef r0
@@ -248,7 +247,7 @@ activityProcessPreempting act s a =
                     modifyIORef' (activityTotalPreemptionTimeRef act) (+ dt)
                     modifyIORef' (activityPreemptionTimeRef act) $
                       addSamplingStats dt
-               triggerSignal (activityReenteringSource act) a 
+               triggerSignal (activityPreemptionEndingSource act) a 
      let m1 =
            do (s', b) <- activityProcess act s a
               dt <- liftIO $ readIORef rs
@@ -335,7 +334,7 @@ activityTotalPreemptionTimeChanged act =
 -- | Signal when the 'activityTotalPreemptionTime' property value has changed.
 activityTotalPreemptionTimeChanged_ :: Activity s a b -> Signal ()
 activityTotalPreemptionTimeChanged_ act =
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
 
 -- | Return the statistics for the time when the activity was utilised.
 --
@@ -396,7 +395,7 @@ activityPreemptionTimeChanged act =
 -- | Signal when the 'activityPreemptionTime' property value has changed.
 activityPreemptionTimeChanged_ :: Activity s a b -> Signal ()
 activityPreemptionTimeChanged_ act =
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
   
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the activity was utilised.
@@ -429,7 +428,7 @@ activityUtilisationFactorChanged_ :: Activity s a b -> Signal ()
 activityUtilisationFactorChanged_ act =
   mapSignal (const ()) (activityUtilising act) <>
   mapSignal (const ()) (activityUtilised act) <>
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
   
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the activity was idle.
@@ -462,7 +461,7 @@ activityIdleFactorChanged_ :: Activity s a b -> Signal ()
 activityIdleFactorChanged_ act =
   mapSignal (const ()) (activityUtilising act) <>
   mapSignal (const ()) (activityUtilised act) <>
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the activity was preempted waiting for the further proceeding.
@@ -495,7 +494,7 @@ activityPreemptionFactorChanged_ :: Activity s a b -> Signal ()
 activityPreemptionFactorChanged_ act =
   mapSignal (const ()) (activityUtilising act) <>
   mapSignal (const ()) (activityUtilised act) <>
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
   
 -- | Raised when starting to utilise the activity after a new input task is received.
 activityUtilising :: Activity s a b -> Signal a
@@ -505,20 +504,20 @@ activityUtilising = publishSignal . activityUtilisingSource
 activityUtilised :: Activity s a b -> Signal (a, b)
 activityUtilised = publishSignal . activityUtilisedSource
 
--- | Raised when the task utilisation by the activity was preempted.
-activityPreempting :: Activity s a b -> Signal a
-activityPreempting = publishSignal . activityPreemptingSource
+-- | Raised when the activity utilisation was preempted.
+activityPreemptionBeginning :: Activity s a b -> Signal a
+activityPreemptionBeginning = publishSignal . activityPreemptionBeginningSource
 
--- | Raised when the task utilisation by the activity was proceeded after it had been preempted earlier.
-activityReentering :: Activity s a b -> Signal a
-activityReentering = publishSignal . activityReenteringSource
+-- | Raised when the activity utilisation was proceeded after it had been preempted earlier.
+activityPreemptionEnding :: Activity s a b -> Signal a
+activityPreemptionEnding = publishSignal . activityPreemptionEndingSource
 
 -- | Signal whenever any property of the activity changes.
 activityChanged_ :: Activity s a b -> Signal ()
 activityChanged_ act =
   mapSignal (const ()) (activityUtilising act) <>
   mapSignal (const ()) (activityUtilised act) <>
-  mapSignal (const ()) (activityReentering act)
+  mapSignal (const ()) (activityPreemptionEnding act)
 
 -- | Return the summary for the activity with desciption of its
 -- properties using the specified indent.
