@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- The module defines a 'Net' arrow that can be applied to modeling the queue networks
 -- like the 'Processor' arrow from another module. Only the former has a more efficient
@@ -60,7 +60,7 @@ import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Stream
 import Simulation.Aivika.QueueStrategy
-import Simulation.Aivika.Resource
+import Simulation.Aivika.Resource.Base
 import Simulation.Aivika.Processor
 import Simulation.Aivika.Ref
 import Simulation.Aivika.Circuit
