@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- This module defines the resource which can be acquired and 
 -- then released by the discontinuous process 'Process'.
@@ -35,6 +35,13 @@ module Simulation.Aivika.Resource
         resourceStrategy,
         resourceMaxCount,
         resourceCount,
+        resourceCountStats,
+        resourceUtilisationCount,
+        resourceUtilisationCountStats,
+        resourceQueueCount,
+        resourceQueueCountStats,
+        resourceTotalWaitTime,
+        resourceWaitTime,
         -- * Requesting for and Releasing Resource
         requestResource,
         requestResourceWithPriority,
@@ -44,9 +51,22 @@ module Simulation.Aivika.Resource
         usingResource,
         usingResourceWithPriority,
         -- * Altering Resource
-        incResourceCount) where
+        incResourceCount,
+        decResourceCount,
+        -- * Signals
+        resourceCountChanged,
+        resourceCountChanged_,
+        resourceUtilisationCountChanged,
+        resourceUtilisationCountChanged_,
+        resourceQueueCountChanged,
+        resourceQueueCountChanged_,
+        resourceWaitTimeChanged,
+        resourceWaitTimeChanged_,
+        resourceChanged_) where
 
 import Data.IORef
+import Data.Monoid
+
 import Control.Monad
 import Control.Monad.Trans
 import Control.Exception
@@ -57,6 +77,8 @@ import Simulation.Aivika.Internal.Event
 import Simulation.Aivika.Internal.Cont
 import Simulation.Aivika.Internal.Process
 import Simulation.Aivika.QueueStrategy
+import Simulation.Aivika.Statistics
+import Simulation.Aivika.Signal
 
 import qualified Simulation.Aivika.DoubleLinkedList as DLL 
 import qualified Simulation.Aivika.Vector as V
@@ -81,8 +103,23 @@ data Resource s =
              resourceMaxCount :: Maybe Int,
              -- ^ Return the maximum count of the resource, where 'Nothing'
              -- means that the resource has no upper bound.
-             resourceCountRef :: IORef Int, 
-             resourceWaitList :: StrategyQueue s (FrozenCont ()) }
+             resourceCountRef :: IORef Int,
+             resourceCountStatsRef :: IORef (TimingStats Int),
+             resourceCountSource :: SignalSource Int,
+             resourceUtilisationCountRef :: IORef Int,
+             resourceUtilisationCountStatsRef :: IORef (TimingStats Int),
+             resourceUtilisationCountSource :: SignalSource Int,
+             resourceQueueCountRef :: IORef Int,
+             resourceQueueCountStatsRef :: IORef (TimingStats Int),
+             resourceQueueCountSource :: SignalSource Int,
+             resourceTotalWaitTimeRef :: IORef Double,
+             resourceWaitTimeRef :: IORef (SamplingStats Double),
+             resourceWaitTimeSource :: SignalSource (),
+             resourceWaitList :: StrategyQueue s ResourceItem }
+
+data ResourceItem =
+  ResourceItem { resourceItemTime :: Double,
+                 resourceItemCont :: FrozenCont () }
 
 instance Eq (Resource s) where
   x == y = resourceCountRef x == resourceCountRef y  -- unique references
@@ -91,7 +128,7 @@ instance Eq (Resource s) where
 -- the upper bound as well.
 newFCFSResource :: Int
                    -- ^ the initial count (and maximal count too) of the resource
-                   -> Simulation FCFSResource
+                   -> Event FCFSResource
 newFCFSResource = newResource FCFS
 
 -- | Create a new FCFS resource with the specified initial and maximum counts,
@@ -100,14 +137,14 @@ newFCFSResourceWithMaxCount :: Int
                                -- ^ the initial count of the resource
                                -> Maybe Int
                                -- ^ the maximum count of the resource, which can be indefinite
-                               -> Simulation FCFSResource
+                               -> Event FCFSResource
 newFCFSResourceWithMaxCount = newResourceWithMaxCount FCFS
 
 -- | Create a new LCFS resource with the specified initial count which value becomes
 -- the upper bound as well.
 newLCFSResource :: Int
                    -- ^ the initial count (and maximal count too) of the resource
-                   -> Simulation LCFSResource
+                   -> Event LCFSResource
 newLCFSResource = newResource LCFS
 
 -- | Create a new LCFS resource with the specified initial and maximum counts,
@@ -116,14 +153,14 @@ newLCFSResourceWithMaxCount :: Int
                                -- ^ the initial count of the resource
                                -> Maybe Int
                                -- ^ the maximum count of the resource, which can be indefinite
-                               -> Simulation LCFSResource
+                               -> Event LCFSResource
 newLCFSResourceWithMaxCount = newResourceWithMaxCount LCFS
 
 -- | Create a new SIRO resource with the specified initial count which value becomes
 -- the upper bound as well.
 newSIROResource :: Int
                    -- ^ the initial count (and maximal count too) of the resource
-                   -> Simulation SIROResource
+                   -> Event SIROResource
 newSIROResource = newResource SIRO
 
 -- | Create a new SIRO resource with the specified initial and maximum counts,
@@ -132,14 +169,14 @@ newSIROResourceWithMaxCount :: Int
                                -- ^ the initial count of the resource
                                -> Maybe Int
                                -- ^ the maximum count of the resource, which can be indefinite
-                               -> Simulation SIROResource
+                               -> Event SIROResource
 newSIROResourceWithMaxCount = newResourceWithMaxCount SIRO
 
 -- | Create a new priority resource with the specified initial count which value becomes
 -- the upper bound as well.
 newPriorityResource :: Int
                        -- ^ the initial count (and maximal count too) of the resource
-                       -> Simulation PriorityResource
+                       -> Event PriorityResource
 newPriorityResource = newResource StaticPriorities
 
 -- | Create a new priority resource with the specified initial and maximum counts,
@@ -148,7 +185,7 @@ newPriorityResourceWithMaxCount :: Int
                                    -- ^ the initial count of the resource
                                    -> Maybe Int
                                    -- ^ the maximum count of the resource, which can be indefinite
-                                   -> Simulation PriorityResource
+                                   -> Event PriorityResource
 newPriorityResourceWithMaxCount = newResourceWithMaxCount StaticPriorities
 
 -- | Create a new resource with the specified queue strategy and initial count.
@@ -158,19 +195,9 @@ newResource :: QueueStrategy s
                -- ^ the strategy for managing the queuing requests
                -> Int
                -- ^ the initial count (and maximal count too) of the resource
-               -> Simulation (Resource s)
+               -> Event (Resource s)
 newResource s count =
-  Simulation $ \r ->
-  do when (count < 0) $
-       error $
-       "The resource count cannot be negative: " ++
-       "newResource."
-     countRef <- newIORef count
-     waitList <- invokeSimulation r $ newStrategyQueue s
-     return Resource { resourceStrategy = s,
-                       resourceMaxCount = Just count,
-                       resourceCountRef = countRef,
-                       resourceWaitList = waitList }
+  newResourceWithMaxCount s count (Just count)
 
 -- | Create a new resource with the specified queue strategy, initial and maximum counts,
 -- where 'Nothing' means that the resource has no upper bound.
@@ -181,10 +208,12 @@ newResourceWithMaxCount :: QueueStrategy s
                            -- ^ the initial count of the resource
                            -> Maybe Int
                            -- ^ the maximum count of the resource, which can be indefinite
-                           -> Simulation (Resource s)
+                           -> Event (Resource s)
 newResourceWithMaxCount s count maxCount =
-  Simulation $ \r ->
-  do when (count < 0) $
+  Event $ \p ->
+  do let r = pointRun p
+         t = pointTime p
+     when (count < 0) $
        error $
        "The resource count cannot be negative: " ++
        "newResourceWithMaxCount."
@@ -196,17 +225,114 @@ newResourceWithMaxCount s count maxCount =
        _ ->
          return ()
      countRef <- newIORef count
+     countStatsRef <- newIORef $ returnTimingStats t count
+     countSource <- invokeSimulation r newSignalSource
+     utilCountRef <- newIORef 0
+     utilCountStatsRef <- newIORef $ returnTimingStats t 0
+     utilCountSource <- invokeSimulation r newSignalSource
+     queueCountRef <- newIORef 0
+     queueCountStatsRef <- newIORef $ returnTimingStats t 0
+     queueCountSource <- invokeSimulation r newSignalSource
+     totalWaitTimeRef <- newIORef 0
+     waitTimeRef <- newIORef emptySamplingStats
+     waitTimeSource <- invokeSimulation r newSignalSource
      waitList <- invokeSimulation r $ newStrategyQueue s
      return Resource { resourceStrategy = s,
                        resourceMaxCount = maxCount,
                        resourceCountRef = countRef,
+                       resourceCountStatsRef = countStatsRef,
+                       resourceCountSource = countSource,
+                       resourceUtilisationCountRef = utilCountRef,
+                       resourceUtilisationCountStatsRef = utilCountStatsRef,
+                       resourceUtilisationCountSource = utilCountSource,
+                       resourceQueueCountRef = queueCountRef,
+                       resourceQueueCountStatsRef = queueCountStatsRef,
+                       resourceQueueCountSource = queueCountSource,
+                       resourceTotalWaitTimeRef = totalWaitTimeRef,
+                       resourceWaitTimeRef = waitTimeRef,
+                       resourceWaitTimeSource = waitTimeSource,
                        resourceWaitList = waitList }
 
--- | Return the current count of the resource.
+-- | Return the current available count of the resource.
 resourceCount :: Resource s -> Event Int
 resourceCount r =
   Event $ \p -> readIORef (resourceCountRef r)
 
+-- | Return the statistics for the available count of the resource.
+resourceCountStats :: Resource s -> Event (TimingStats Int)
+resourceCountStats r =
+  Event $ \p -> readIORef (resourceCountStatsRef r)
+
+-- | Signal triggered when the 'resourceCount' property changes.
+resourceCountChanged :: Resource s -> Signal Int
+resourceCountChanged r =
+  publishSignal $ resourceCountSource r
+
+-- | Signal triggered when the 'resourceCount' property changes.
+resourceCountChanged_ :: Resource s -> Signal ()
+resourceCountChanged_ r =
+  mapSignal (const ()) $ resourceCountChanged r
+
+-- | Return the current utilisation count of the resource.
+resourceUtilisationCount :: Resource s -> Event Int
+resourceUtilisationCount r =
+  Event $ \p -> readIORef (resourceUtilisationCountRef r)
+
+-- | Return the statistics for the utilisation count of the resource.
+resourceUtilisationCountStats :: Resource s -> Event (TimingStats Int)
+resourceUtilisationCountStats r =
+  Event $ \p -> readIORef (resourceUtilisationCountStatsRef r)
+
+-- | Signal triggered when the 'resourceUtilisationCount' property changes.
+resourceUtilisationCountChanged :: Resource s -> Signal Int
+resourceUtilisationCountChanged r =
+  publishSignal $ resourceUtilisationCountSource r
+
+-- | Signal triggered when the 'resourceUtilisationCount' property changes.
+resourceUtilisationCountChanged_ :: Resource s -> Signal ()
+resourceUtilisationCountChanged_ r =
+  mapSignal (const ()) $ resourceUtilisationCountChanged r
+
+-- | Return the current queue length of the resource.
+resourceQueueCount :: Resource s -> Event Int
+resourceQueueCount r =
+  Event $ \p -> readIORef (resourceQueueCountRef r)
+
+-- | Return the statistics for the queue length of the resource.
+resourceQueueCountStats :: Resource s -> Event (TimingStats Int)
+resourceQueueCountStats r =
+  Event $ \p -> readIORef (resourceQueueCountStatsRef r)
+
+-- | Signal triggered when the 'resourceQueueCount' property changes.
+resourceQueueCountChanged :: Resource s -> Signal Int
+resourceQueueCountChanged r =
+  publishSignal $ resourceQueueCountSource r
+
+-- | Signal triggered when the 'resourceQueueCount' property changes.
+resourceQueueCountChanged_ :: Resource s -> Signal ()
+resourceQueueCountChanged_ r =
+  mapSignal (const ()) $ resourceQueueCountChanged r
+
+-- | Return the total wait time of the resource.
+resourceTotalWaitTime :: Resource s -> Event Double
+resourceTotalWaitTime r =
+  Event $ \p -> readIORef (resourceTotalWaitTimeRef r)
+
+-- | Return the statistics for the wait time of the resource.
+resourceWaitTime :: Resource s -> Event (SamplingStats Double)
+resourceWaitTime r =
+  Event $ \p -> readIORef (resourceWaitTimeRef r)
+
+-- | Signal triggered when the 'resourceTotalWaitTime' and 'resourceWaitTime' properties change.
+resourceWaitTimeChanged :: Resource s -> Signal (SamplingStats Double)
+resourceWaitTimeChanged r =
+  mapSignalM (\() -> resourceWaitTime r) $ resourceWaitTimeChanged_ r
+
+-- | Signal triggered when the 'resourceTotalWaitTime' and 'resourceWaitTime' properties change.
+resourceWaitTimeChanged_ :: Resource s -> Signal ()
+resourceWaitTimeChanged_ r =
+  publishSignal $ resourceWaitTimeSource r
+
 -- | Request for the resource decreasing its count in case of success,
 -- otherwise suspending the discontinuous process until some other 
 -- process releases the resource.
@@ -226,9 +352,12 @@ requestResource r =
                     invokeProcess pid $
                     requestResource r
                invokeEvent p $
-                 strategyEnqueue (resourceWaitList r) c
-       else do let a' = a - 1
-               a' `seq` writeIORef (resourceCountRef r) a'
+                 strategyEnqueue (resourceWaitList r) $
+                 ResourceItem (pointTime p) c
+               invokeEvent p $ updateResourceQueueCount r 1
+       else do invokeEvent p $ updateResourceWaitTime r 0
+               invokeEvent p $ updateResourceCount r (-1)
+               invokeEvent p $ updateResourceUtilisationCount r 1
                invokeEvent p $ resumeCont c ()
 
 -- | Request with the priority for the resource decreasing its count
@@ -252,9 +381,12 @@ requestResourceWithPriority r priority =
                     invokeProcess pid $
                     requestResourceWithPriority r priority
                invokeEvent p $
-                 strategyEnqueueWithPriority (resourceWaitList r) priority c
-       else do let a' = a - 1
-               a' `seq` writeIORef (resourceCountRef r) a'
+                 strategyEnqueueWithPriority (resourceWaitList r) priority $
+                 ResourceItem (pointTime p) c
+               invokeEvent p $ updateResourceQueueCount r 1
+       else do invokeEvent p $ updateResourceWaitTime r 0
+               invokeEvent p $ updateResourceCount r (-1)
+               invokeEvent p $ updateResourceUtilisationCount r 1
                invokeEvent p $ resumeCont c ()
 
 -- | Release the resource increasing its count and resuming one of the
@@ -277,6 +409,16 @@ releaseResourceWithinEvent :: DequeueStrategy s
                               -- ^ the resource to release
                               -> Event ()
 releaseResourceWithinEvent r =
+  Event $ \p ->
+  do invokeEvent p $ updateResourceUtilisationCount r (-1)
+     invokeEvent p $ releaseResource' r
+  
+-- | Release the resource without affecting its utilisation.
+releaseResource' :: DequeueStrategy s
+                    => Resource s
+                    -- ^ the resource to release
+                    -> Event ()
+releaseResource' r =
   Event $ \p ->
   do a <- readIORef (resourceCountRef r)
      let a' = a + 1
@@ -284,21 +426,24 @@ releaseResourceWithinEvent r =
        Just maxCount | a' > maxCount ->
          error $
          "The resource count cannot be greater than " ++
-         "its maximum value: releaseResourceWithinEvent."
+         "its maximum value: releaseResource'."
        _ ->
          return ()
      f <- invokeEvent p $
           strategyQueueNull (resourceWaitList r)
      if f 
-       then a' `seq` writeIORef (resourceCountRef r) a'
-       else do c <- invokeEvent p $
+       then invokeEvent p $ updateResourceCount r 1
+       else do x <- invokeEvent p $
                     strategyDequeue (resourceWaitList r)
-               c <- invokeEvent p $ unfreezeCont c
+               invokeEvent p $ updateResourceQueueCount r (-1)
+               c <- invokeEvent p $ unfreezeCont (resourceItemCont x)
                case c of
                  Nothing ->
-                   invokeEvent p $ releaseResourceWithinEvent r
+                   invokeEvent p $ releaseResource' r
                  Just c  ->
-                   invokeEvent p $ enqueueEvent (pointTime p) $ resumeCont c ()
+                   do invokeEvent p $ updateResourceWaitTime r (pointTime p - resourceItemTime x)
+                      invokeEvent p $ updateResourceUtilisationCount r 1
+                      invokeEvent p $ enqueueEvent (pointTime p) $ resumeCont c ()
 
 -- | Try to request for the resource decreasing its count in case of success
 -- and returning 'True' in the 'Event' monad; otherwise, returning 'False'.
@@ -310,8 +455,9 @@ tryRequestResourceWithinEvent r =
   do a <- readIORef (resourceCountRef r)
      if a == 0 
        then return False
-       else do let a' = a - 1
-               a' `seq` writeIORef (resourceCountRef r) a'
+       else do invokeEvent p $ updateResourceWaitTime r 0
+               invokeEvent p $ updateResourceCount r (-1)
+               invokeEvent p $ updateResourceUtilisationCount r 1
                return True
                
 -- | Acquire the resource, perform some action and safely release the resource               
@@ -344,6 +490,16 @@ usingResourceWithPriority r priority m =
   do requestResourceWithPriority r priority
      finallyProcess m $ releaseResource r
 
+-- | Decrease the count of available resource.
+decResourceCount' :: EnqueueStrategy s
+                     => Resource s
+                     -- ^ the resource for which to decrease the count
+                     -> Process ()
+decResourceCount' r =
+  do liftEvent $
+       updateResourceUtilisationCount r (-1)
+     requestResource r
+                   
 -- | Increase the count of available resource by the specified number,
 -- invoking the awaiting processes as needed.
 incResourceCount :: DequeueStrategy s
@@ -356,6 +512,75 @@ incResourceCount r n
   | n < 0     = error "The increment cannot be negative: incResourceCount"
   | n == 0    = return ()
   | otherwise =
-    do releaseResourceWithinEvent r
+    do releaseResource' r
        incResourceCount r (n - 1)
 
+-- | Decrease the count of available resource by the specified number,
+-- waiting for the processes capturing the resource as needed.
+decResourceCount :: EnqueueStrategy s
+                    => Resource s
+                    -- ^ the resource
+                    -> Int
+                    -- ^ the decrement for the resource count
+                    -> Process ()
+decResourceCount r n
+  | n < 0     = error "The decrement cannot be negative: decResourceCount"
+  | n == 0    = return ()
+  | otherwise =
+    do decResourceCount' r
+       decResourceCount r (n - 1)
+
+-- | Signal triggered when one of the resource counters changes.
+resourceChanged_ :: Resource s -> Signal ()
+resourceChanged_ r =
+  resourceCountChanged_ r <>
+  resourceUtilisationCountChanged_ r <>
+  resourceQueueCountChanged_ r
+
+-- | Update the resource count and its statistics.
+updateResourceCount :: Resource s -> Int -> Event ()
+updateResourceCount r delta =
+  Event $ \p ->
+  do a <- readIORef (resourceCountRef r)
+     let a' = a + delta
+     a' `seq` writeIORef (resourceCountRef r) a'
+     modifyIORef' (resourceCountStatsRef r) $
+       addTimingStats (pointTime p) a'
+     invokeEvent p $
+       triggerSignal (resourceCountSource r) a'
+
+-- | Update the resource utilisation count and its statistics.
+updateResourceUtilisationCount :: Resource s -> Int -> Event ()
+updateResourceUtilisationCount r delta =
+  Event $ \p ->
+  do a <- readIORef (resourceUtilisationCountRef r)
+     let a' = a + delta
+     a' `seq` writeIORef (resourceUtilisationCountRef r) a'
+     modifyIORef' (resourceUtilisationCountStatsRef r) $
+       addTimingStats (pointTime p) a'
+     invokeEvent p $
+       triggerSignal (resourceUtilisationCountSource r) a'
+
+-- | Update the resource queue length and its statistics.
+updateResourceQueueCount :: Resource s -> Int -> Event ()
+updateResourceQueueCount r delta =
+  Event $ \p ->
+  do a <- readIORef (resourceQueueCountRef r)
+     let a' = a + delta
+     a' `seq` writeIORef (resourceQueueCountRef r) a'
+     modifyIORef' (resourceQueueCountStatsRef r) $
+       addTimingStats (pointTime p) a'
+     invokeEvent p $
+       triggerSignal (resourceQueueCountSource r) a'
+
+-- | Update the resource wait time and its statistics.
+updateResourceWaitTime :: Resource s -> Double -> Event ()
+updateResourceWaitTime r delta =
+  Event $ \p ->
+  do a <- readIORef (resourceTotalWaitTimeRef r)
+     let a' = a + delta
+     a' `seq` writeIORef (resourceTotalWaitTimeRef r) a'
+     modifyIORef' (resourceWaitTimeRef r) $
+       addSamplingStats delta
+     invokeEvent p $
+       triggerSignal (resourceWaitTimeSource r) ()
