@@ -7,7 +7,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- The module allows exporting the simulation results from the model.
 --
@@ -124,7 +124,7 @@ import Simulation.Aivika.Signal
 import Simulation.Aivika.Statistics
 import Simulation.Aivika.Statistics.Accumulator
 import Simulation.Aivika.Ref
-import qualified Simulation.Aivika.Ref.Plain as LR
+import qualified Simulation.Aivika.Ref.Base as BR
 import Simulation.Aivika.Var
 import Simulation.Aivika.QueueStrategy
 import qualified Simulation.Aivika.Queue as Q
@@ -132,6 +132,8 @@ import qualified Simulation.Aivika.Queue.Infinite as IQ
 import Simulation.Aivika.Arrival
 import Simulation.Aivika.Server
 import Simulation.Aivika.Activity
+import Simulation.Aivika.Resource
+import qualified Simulation.Aivika.Resource.Preemption as PR
 import Simulation.Aivika.Results.Locale
 
 -- | A name used for indentifying the results when generating output.
@@ -432,6 +434,10 @@ data ResultValue e =
 instance Functor ResultValue where
   fmap f x = x { resultValueData = fmap f (resultValueData x) }
 
+-- | Transform the result value.
+apResultValue :: ResultData (a -> b) -> ResultValue a -> ResultValue b
+apResultValue f x = x { resultValueData = ap f (resultValueData x) }
+
 -- | A container of the simulation results such as queue, server or array.
 data ResultContainer e =
   ResultContainer { resultContainerName :: ResultName,
@@ -564,6 +570,12 @@ resultContainerToValue x =
 -- | Represents the very simulation results.
 type ResultData e = Event e
 
+-- | Convert the timing statistics data to its normalised sampling-based representation.
+normTimingStatsData :: TimingData a => ResultData (TimingStats a -> SamplingStats a)
+normTimingStatsData =
+  do n <- liftDynamics integIteration
+     return $ normTimingStats (fromIntegral n)
+
 -- | Whether an object containing the results emits a signal notifying about change of data.
 data ResultSignal = EmptyResultSignal
                     -- ^ There is no signal at all.
@@ -734,12 +746,12 @@ instance ResultItemable (ResultValue (TimingStats Int)) where
   
   resultItemAsIntValue = const Nothing
   resultItemAsIntListValue = const Nothing
-  resultItemAsIntStatsValue = const Nothing
+  resultItemAsIntStatsValue = Just . apResultValue normTimingStatsData
   resultItemAsIntTimingStatsValue = Just
 
   resultItemAsDoubleValue = const Nothing
   resultItemAsDoubleListValue = const Nothing
-  resultItemAsDoubleStatsValue = const Nothing
+  resultItemAsDoubleStatsValue = Just . fmap fromIntSamplingStats . apResultValue normTimingStatsData
   resultItemAsDoubleTimingStatsValue = Just . fmap fromIntTimingStats
 
   resultItemAsStringValue = Just . fmap show
@@ -760,7 +772,7 @@ instance ResultItemable (ResultValue (TimingStats Double)) where
 
   resultItemAsDoubleValue = const Nothing
   resultItemAsDoubleListValue = const Nothing
-  resultItemAsDoubleStatsValue = const Nothing
+  resultItemAsDoubleStatsValue = Just . apResultValue normTimingStatsData
   resultItemAsDoubleTimingStatsValue = Just
 
   resultItemAsStringValue = Just . fmap show
@@ -1319,9 +1331,9 @@ instance ResultComputing Ref where
   computeResultData = readRef
   computeResultSignal = ResultSignal . refChanged_
 
-instance ResultComputing LR.Ref where
+instance ResultComputing BR.Ref where
 
-  computeResultData = LR.readRef
+  computeResultData = BR.readRef
   computeResultSignal = const UnknownResultSignal
 
 instance ResultComputing Var where
@@ -1729,6 +1741,87 @@ activityResultSummary c =
       resultContainerProperty c "idleFactor" ActivityIdleFactorId activityIdleFactor activityIdleFactorChanged_,
       resultContainerProperty c "preemptionFactor" ActivityPreemptionFactorId activityPreemptionFactor activityPreemptionFactorChanged_ ] }
 
+-- | Return a source by the specified resource.
+resourceResultSource :: (Show s, ResultItemable (ResultValue s))
+                        => ResultContainer (Resource s)
+                        -- ^ the resource container
+                        -> ResultSource
+resourceResultSource c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = ResourceId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = resourceResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "queueCount" ResourceQueueCountId resourceQueueCount resourceQueueCountChanged_,
+      resultContainerProperty c "queueCountStats" ResourceQueueCountStatsId resourceQueueCountStats resourceQueueCountChanged_,
+      resultContainerProperty c "totalWaitTime" ResourceTotalWaitTimeId resourceTotalWaitTime resourceWaitTimeChanged_,
+      resultContainerProperty c "waitTime" ResourceWaitTimeId resourceWaitTime resourceWaitTimeChanged_,
+      resultContainerProperty c "count" ResourceCountId resourceCount resourceCountChanged_,
+      resultContainerProperty c "countStats" ResourceCountStatsId resourceCountStats resourceCountChanged_,
+      resultContainerProperty c "utilisationCount" ResourceUtilisationCountId resourceUtilisationCount resourceUtilisationCountChanged_,
+      resultContainerProperty c "utilisationCountStats" ResourceUtilisationCountStatsId resourceUtilisationCountStats resourceUtilisationCountChanged_ ] }
+
+-- | Return a summary by the specified resource.
+resourceResultSummary :: ResultContainer (Resource s)
+                         -- ^ the resource container
+                         -> ResultSource
+resourceResultSummary c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = ResourceId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = resourceResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "queueCountStats" ResourceQueueCountStatsId resourceQueueCountStats resourceQueueCountChanged_,
+      resultContainerProperty c "waitTime" ResourceWaitTimeId resourceWaitTime resourceWaitTimeChanged_,
+      resultContainerProperty c "countStats" ResourceCountStatsId resourceCountStats resourceCountChanged_,
+      resultContainerProperty c "utilisationCountStats" ResourceUtilisationCountStatsId resourceUtilisationCountStats resourceUtilisationCountChanged_ ] }
+
+-- | Return a source by the specified resource.
+preemptibleResourceResultSource :: ResultContainer PR.Resource
+                                   -- ^ the resource container
+                                   -> ResultSource
+preemptibleResourceResultSource c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = ResourceId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = preemptibleResourceResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "queueCount" ResourceQueueCountId PR.resourceQueueCount PR.resourceQueueCountChanged_,
+      resultContainerProperty c "queueCountStats" ResourceQueueCountStatsId PR.resourceQueueCountStats PR.resourceQueueCountChanged_,
+      resultContainerProperty c "totalWaitTime" ResourceTotalWaitTimeId PR.resourceTotalWaitTime PR.resourceWaitTimeChanged_,
+      resultContainerProperty c "waitTime" ResourceWaitTimeId PR.resourceWaitTime PR.resourceWaitTimeChanged_,
+      resultContainerProperty c "count" ResourceCountId PR.resourceCount PR.resourceCountChanged_,
+      resultContainerProperty c "countStats" ResourceCountStatsId PR.resourceCountStats PR.resourceCountChanged_,
+      resultContainerProperty c "utilisationCount" ResourceUtilisationCountId PR.resourceUtilisationCount PR.resourceUtilisationCountChanged_,
+      resultContainerProperty c "utilisationCountStats" ResourceUtilisationCountStatsId PR.resourceUtilisationCountStats PR.resourceUtilisationCountChanged_ ] }
+
+-- | Return a summary by the specified resource.
+preemptibleResourceResultSummary :: ResultContainer PR.Resource
+                                    -- ^ the resource container
+                                    -> ResultSource
+preemptibleResourceResultSummary c =
+  ResultObjectSource $
+  ResultObject {
+    resultObjectName = resultContainerName c,
+    resultObjectId = resultContainerId c,
+    resultObjectTypeId = ResourceId,
+    resultObjectSignal = resultContainerSignal c,
+    resultObjectSummary = preemptibleResourceResultSummary c,
+    resultObjectProperties = [
+      resultContainerProperty c "queueCountStats" ResourceQueueCountStatsId PR.resourceQueueCountStats PR.resourceQueueCountChanged_,
+      resultContainerProperty c "waitTime" ResourceWaitTimeId PR.resourceWaitTime PR.resourceWaitTimeChanged_,
+      resultContainerProperty c "countStats" ResourceCountStatsId PR.resourceCountStats PR.resourceCountChanged_,
+      resultContainerProperty c "utilisationCountStats" ResourceUtilisationCountStatsId PR.resourceUtilisationCountStats PR.resourceUtilisationCountChanged_ ] }
+
 -- | Return an arbitrary text as a separator source.
 textResultSource :: String -> ResultSource
 textResultSource text =
@@ -1959,3 +2052,13 @@ instance (Show s, ResultItemable (ResultValue s)) => ResultProvider (Activity s
 
   resultSource' name i m =
     activityResultSource $ ResultContainer name i m (ResultSignal $ activityChanged_ m)
+
+instance (Show s, ResultItemable (ResultValue s)) => ResultProvider (Resource s) where
+
+  resultSource' name i m =
+    resourceResultSource $ ResultContainer name i m (ResultSignal $ resourceChanged_ m)
+
+instance ResultProvider PR.Resource where
+
+  resultSource' name i m =
+    preemptibleResourceResultSource $ ResultContainer name i m (ResultSignal $ PR.resourceChanged_ m)
