@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- It models the server that prodives a service.
 module Simulation.Aivika.Server
@@ -63,8 +63,8 @@ module Simulation.Aivika.Server
         serverPreemptionFactorChanged_,
         -- * Basic Signals
         serverInputReceived,
-        serverTaskPreempting,
-        serverTaskReentering,
+        serverTaskPreemptionBeginning,
+        serverTaskPreemptionEnding,
         serverTaskProcessed,
         serverOutputProvided,
         -- * Overall Signal
@@ -81,7 +81,6 @@ import Simulation.Aivika.Simulation
 import Simulation.Aivika.Dynamics
 import Simulation.Aivika.Internal.Event
 import Simulation.Aivika.Signal
-import Simulation.Aivika.Resource
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Processor
@@ -116,9 +115,9 @@ data Server s a b =
            -- ^ The statistics for the time spent being preempted.
            serverInputReceivedSource :: SignalSource a,
            -- ^ A signal raised when the server recieves a new input to process.
-           serverTaskPreemptingSource :: SignalSource a,
+           serverTaskPreemptionBeginningSource :: SignalSource a,
            -- ^ A signal raised when the task was preempted.
-           serverTaskReenteringSource :: SignalSource a,
+           serverTaskPreemptionEndingSource :: SignalSource a,
            -- ^ A signal raised when the task was proceeded after it had been preempted earlier.
            serverTaskProcessedSource :: SignalSource (a, b),
            -- ^ A signal raised when the input is processed and
@@ -200,8 +199,8 @@ newPreemptibleStateServer preemptible provide state =
                            serverOutputWaitTimeRef = r7,
                            serverPreemptionTimeRef = r8,
                            serverInputReceivedSource = s1,
-                           serverTaskPreemptingSource = s2,
-                           serverTaskReenteringSource = s3,
+                           serverTaskPreemptionBeginningSource = s2,
+                           serverTaskPreemptionEndingSource = s3,
                            serverTaskProcessedSource = s4,
                            serverOutputProvidedSource = s5 }
      return server
@@ -279,7 +278,7 @@ serverProcessPreempting server s a =
             handleSignal (processPreemptionBeginning pid) $ \() ->
             do t1 <- liftDynamics time
                liftIO $ writeIORef r1 t1
-               triggerSignal (serverTaskPreemptingSource server) a
+               triggerSignal (serverTaskPreemptionBeginningSource server) a
      h2  <- liftEvent $
             handleSignal (processPreemptionEnding pid) $ \() ->
             do t1 <- liftIO $ readIORef r1
@@ -290,7 +289,7 @@ serverProcessPreempting server s a =
                     modifyIORef' (serverTotalPreemptionTimeRef server) (+ dt)
                     modifyIORef' (serverPreemptionTimeRef server) $
                       addSamplingStats dt
-               triggerSignal (serverTaskReenteringSource server) a 
+               triggerSignal (serverTaskPreemptionEndingSource server) a 
      let m1 =
            do (s', b) <- serverProcess server s a
               dt <- liftIO $ readIORef rs
@@ -398,7 +397,7 @@ serverTotalPreemptionTimeChanged server =
 -- | Signal when the 'serverTotalPreemptionTime' property value has changed.
 serverTotalPreemptionTimeChanged_ :: Server s a b -> Signal ()
 serverTotalPreemptionTimeChanged_ server =
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | Return the statistics of the time when the server was locked while awaiting the input.
 --
@@ -480,7 +479,7 @@ serverPreemptionTimeChanged server =
 -- | Signal when the 'serverPreemptionTime' property value has changed.
 serverPreemptionTimeChanged_ :: Server s a b -> Signal ()
 serverPreemptionTimeChanged_ server =
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was awaiting for the next input task.
@@ -515,7 +514,7 @@ serverInputWaitFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
   mapSignal (const ()) (serverOutputProvided server) <>
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was busy with direct processing its tasks.
@@ -550,7 +549,7 @@ serverProcessingFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
   mapSignal (const ()) (serverOutputProvided server) <>
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was locked trying to deliver the output after the task is finished.
@@ -585,7 +584,7 @@ serverOutputWaitFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
   mapSignal (const ()) (serverOutputProvided server) <>
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | It returns the factor changing from 0 to 1, which estimates how often
 -- the server was preempted waiting for the further proceeding.
@@ -620,19 +619,19 @@ serverPreemptionFactorChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
   mapSignal (const ()) (serverOutputProvided server) <>
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | Raised when the server receives a new input task.
 serverInputReceived :: Server s a b -> Signal a
 serverInputReceived = publishSignal . serverInputReceivedSource
 
--- | Raised when the task processing by the server was preempted.
-serverTaskPreempting :: Server s a b -> Signal a
-serverTaskPreempting = publishSignal . serverTaskPreemptingSource
+-- | Raised when the task processing was preempted.
+serverTaskPreemptionBeginning :: Server s a b -> Signal a
+serverTaskPreemptionBeginning = publishSignal . serverTaskPreemptionBeginningSource
 
--- | Raised when the task processing by the server was proceeded after it has been preempeted earlier.
-serverTaskReentering :: Server s a b -> Signal a
-serverTaskReentering = publishSignal . serverTaskReenteringSource
+-- | Raised when the task processing was proceeded after it had been preempeted earlier.
+serverTaskPreemptionEnding :: Server s a b -> Signal a
+serverTaskPreemptionEnding = publishSignal . serverTaskPreemptionEndingSource
 
 -- | Raised when the server has just processed the task.
 serverTaskProcessed :: Server s a b -> Signal (a, b)
@@ -648,7 +647,7 @@ serverChanged_ server =
   mapSignal (const ()) (serverInputReceived server) <>
   mapSignal (const ()) (serverTaskProcessed server) <>
   mapSignal (const ()) (serverOutputProvided server) <>
-  mapSignal (const ()) (serverTaskReentering server)
+  mapSignal (const ()) (serverTaskPreemptionEnding server)
 
 -- | Return the summary for the server with desciption of its
 -- properties and activities using the specified indent.
