@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- Represents statistics.
 --
@@ -28,19 +28,20 @@ module Simulation.Aivika.Statistics
         timingStatsSummary,
         returnTimingStats,
         fromIntTimingStats,
+        normTimingStats,
         -- * Simple Counter
         SamplingCounter(..),
         emptySamplingCounter,
         incSamplingCounter,
         decSamplingCounter,
-        resetSamplingCounter,
+        setSamplingCounter,
         returnSamplingCounter,
         -- * Timing Counter
         TimingCounter(..),
         emptyTimingCounter,
         incTimingCounter,
         decTimingCounter,
-        resetTimingCounter,
+        setTimingCounter,
         returnTimingCounter) where
 
 import Data.Monoid
@@ -272,6 +273,9 @@ class Num a => TimingData a where
   -- | Return the average value.
   timingStatsMean :: TimingStats a -> Double
   
+  -- | Return the average square value.
+  timingStatsMean2 :: TimingStats a -> Double
+  
   -- | Return the variance.
   timingStatsVariance :: TimingStats a -> Double
   
@@ -291,6 +295,7 @@ instance TimingData Double where
     
   addTimingStats      = addTimingStatsGeneric
   timingStatsMean     = timingStatsMeanGeneric
+  timingStatsMean2    = timingStatsMean2Generic
   timingStatsVariance = timingStatsVarianceGeneric
 
 instance TimingData Int where
@@ -309,6 +314,7 @@ instance TimingData Int where
     
   addTimingStats      = addTimingStatsGeneric
   timingStatsMean     = timingStatsMeanGeneric
+  timingStatsMean2    = timingStatsMean2Generic
   timingStatsVariance = timingStatsVarianceGeneric
 
 addTimingStatsGeneric :: ConvertableToDouble a => Double -> a -> TimingStats a -> TimingStats a
@@ -396,6 +402,16 @@ fromIntTimingStats stats =
           timingStatsMax  = fromIntegral $ timingStatsMax stats,
           timingStatsLast = fromIntegral $ timingStatsLast stats }
 
+-- | Convert the statistics to its normalised sampling-based representation,
+-- where the first argument specifies the number of pseudo-samples.
+normTimingStats :: TimingData a => Int -> TimingStats a -> SamplingStats a
+normTimingStats n stats =
+  SamplingStats { samplingStatsCount = n,
+                  samplingStatsMin   = timingStatsMin stats,
+                  samplingStatsMax   = timingStatsMax stats,
+                  samplingStatsMean  = timingStatsMean stats,
+                  samplingStatsMean2 = timingStatsMean2 stats }
+
 -- | Show the summary of the statistics.       
 showTimingStats :: (Show a, TimingData a) => TimingStats a -> ShowS
 showTimingStats stats =
@@ -467,11 +483,11 @@ decSamplingCounter a counter =
                     samplingCounterStats = addSamplingStats a' (samplingCounterStats counter) }
   where a' = samplingCounterValue counter - a
 
--- | Reset the counter.
-resetSamplingCounter :: SamplingData a => SamplingCounter a -> SamplingCounter a
-resetSamplingCounter counter =
-  SamplingCounter { samplingCounterValue = 0,
-                    samplingCounterStats = addSamplingStats 0 (samplingCounterStats counter) }
+-- | Set a new value for the counter.
+setSamplingCounter :: SamplingData a => a -> SamplingCounter a -> SamplingCounter a
+setSamplingCounter a counter =
+  SamplingCounter { samplingCounterValue = a,
+                    samplingCounterStats = addSamplingStats a (samplingCounterStats counter) }
 
 -- | Create a counter with the specified initial value.
 returnSamplingCounter :: SamplingData  a => a -> SamplingCounter a
@@ -507,11 +523,11 @@ decTimingCounter t a counter =
                   timingCounterStats = addTimingStats t a' (timingCounterStats counter) }
   where a' = timingCounterValue counter - a
 
--- | Reset the counter at the specified time.
-resetTimingCounter :: TimingData a => Double -> TimingCounter a -> TimingCounter a
-resetTimingCounter t counter =
-  TimingCounter { timingCounterValue = 0,
-                  timingCounterStats = addTimingStats t 0 (timingCounterStats counter) }
+-- | Set a new value for the counter at the specified time.
+setTimingCounter :: TimingData a => Double -> a -> TimingCounter a -> TimingCounter a
+setTimingCounter t a counter =
+  TimingCounter { timingCounterValue = a,
+                  timingCounterStats = addTimingStats t a (timingCounterStats counter) }
 
 -- | Create a timing counter with the specified initial value at the given time.
 returnTimingCounter :: TimingData a => Double -> a -> TimingCounter a
