@@ -5,7 +5,7 @@
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
--- Tested with: GHC 7.8.3
+-- Tested with: GHC 7.10.1
 --
 -- The infinite stream of data in time.
 --
@@ -86,7 +86,7 @@ import Simulation.Aivika.Event
 import Simulation.Aivika.Cont
 import Simulation.Aivika.Process
 import Simulation.Aivika.Signal
-import Simulation.Aivika.Resource
+import Simulation.Aivika.Resource.Base
 import Simulation.Aivika.QueueStrategy
 import Simulation.Aivika.Queue.Infinite
 import Simulation.Aivika.Internal.Arrival
